@@ -105,6 +105,7 @@ class Generator(OptimizationObject):
         rampratemax=None,rampratemin=None,
         costcurvestring='20P',
         heatratestring=None,fuelcost=1,
+        bid_points=None,
         startupcost=0,shutdowncost=0,
         mustrun=False,
         name='',index=None,bus=None,
@@ -120,7 +121,7 @@ class Generator(OptimizationObject):
     def power(self,time=None): 
         '''real power output at time'''
         return self.get_variable('power',time,indexed=True)
-    def status(self,time): 
+    def status(self,time=None): 
         '''on/off status at time'''
         if self.commitment_problem: return self.get_variable('status',time,indexed=True)
         else: return 1
@@ -147,20 +148,19 @@ class Generator(OptimizationObject):
         else: 
             c=self.get_variable('shutdowncost',time,indexed=True)
             return c if not evaluate else value(c) 
-    def operatingcost(self,time,evaluate=False): 
+    def operatingcost(self,time=None,evaluate=False): 
         '''cost of real power production at time (based on bid model approximation).'''
-        return self.bid(time).output(evaluate)
+        return self.bids.output(time,evaluate)
     def truecost(self,time):
         '''exact cost of real power production at time (based on exact bid polynomial).'''
-        return value(self.status(time))*self.cost_model.output_true(self.power(time))
+        return value(self.status(time))*self.bids.output_true(self.power(time))
     def incrementalcost(self,time): 
         '''change in cost with change in power at time (based on exact bid polynomial).'''
-        return self.bid(time).output_incremental(self.power(time)) if value(self.status(time)) else None
-    def bid(self,time):
-        ''':class:`~bidding.Bid` object for time'''
-        return self.get_child('bids', time) 
+        return self.bids.output_incremental(self.power(time)) if value(self.status(time)) else None
+    def cost_first_stage(self,times):  return sum(self.cost_startup(time)+self.cost_shutdown(time) for time in times)
+    def cost_second_stage(self,times): return sum(self.operatingcost(time) for time in times)
     def getstatus(self,t,times): return dict(u=value(self.status(t)),P=value(self.power(t)),hoursinstatus=self.gethrsinstatus(t,times))
-    def plot_cost_curve(self,P=None,filename=None): self.cost_model.plot(P,filename)
+#    def plot_cost_curve(self,P=None,filename=None): self.cost_model.plot(P,filename)
     def gethrsinstatus(self,tm,times):
         if not self.is_controllable: return None
         status=value(self.status(tm))
@@ -180,15 +180,26 @@ class Generator(OptimizationObject):
         self.initial_power =P*u #note: this eliminates ambiguity of off status with power non-zero output
         self.initial_status_hours = hoursinstatus
     def build_cost_model(self):
-        ''' create a cost model for bidding with :meth:`~bidding.makeModel` '''
-        if getattr(self,'heatratestring',None) is not None: 
-            costinputs=dict(polyText=self.heatratestring,multiplier=self.fuelcost)
-            self.costcurvestring=None
-        else: 
-            costinputs=dict(polyText=self.costcurvestring)
-            self.fuelcost=1
-        
-        self.cost_model=bidding.makeModel(min_input=self.Pmin, max_input=self.Pmax,input_name='Pg',output_name='C',**costinputs)
+        '''parse the coefficients for the polynomial bid curve'''
+        if self.bid_points is None:
+            # polynomial specification
+            self.cost_breakpoints=config.default_num_breakpoints
+            if getattr(self,'heatratestring',None) is not None: 
+                self.cost_coeffs=[self.fuelcost*mult for mult in bidding.parse_polynomial(self.heatratestring)]
+            else:
+                self.cost_coeffs=bidding.parse_polynomial(self.costcurvestring)
+        else:
+            # do some simple validation and delay construction to bidding object 
+            min_power_bid = self.bid_points[0][0]
+            max_power_bid = self.bid_points[-1][0]
+            if min_power_bid>self.Pmin: 
+                self.Pmin = min_power_bid
+                logging.warning('{g} should have a min. power bid ({mpb}) <= to its min. power limit ({mpl})'.format(g=str(self), mpb=min_power_bid, mpl=self.Pmin))
+                
+            if max_power_bid<self.Pmax: 
+                self.Pmax = max_power_bid
+                logging.warning('{g} should have a max. power bid ({mpb}) >= to its max. power limit ({mpl})'.format(g=str(self), mpb=max_power_bid, mpl=self.Pmax))
+            
 
     def create_variables(self,times):
         '''
@@ -197,29 +208,32 @@ class Generator(OptimizationObject):
         '''
         self.commitment_problem= len(times)>1 or self.dispatch_decommit_allowed
         self.add_variable('power', index=times.set, low=0, high=self.Pmax)
-        
         if self.commitment_problem:
             self.add_variable('status', index=times.set, kind='Binary',fixed_value=1 if self.mustrun else None)
             #only use capacity if reserve req. 
             #self.add_variable('capacity',index=times.set, low=0,high=self.Pmax)
             if self.startupcost>0:  self.add_variable('startupcost',index=times.set, low=0,high=self.startupcost)                                                                                                                            
             if self.shutdowncost>0: self.add_variable('shutdowncost',index=times.set, low=0,high=self.shutdowncost)
-        # else: #ED or OPF problem, no commitments
-            # self.add_variable('status',       index=times.set,fixed_value=1)
-            # self.add_variable('startupcost',  index=times.set,fixed_value=0)
-            # self.add_variable('shutdowncost', index=times.set,fixed_value=0)
 
-        bids=dict(zip(times,[bidding.Bid(
-                    model=self.cost_model,
-                    time=time,
-                    input_var=self.power(time),
-                    status_var=self.status(time),
-                    owner_iden=str(self),
-                    time_iden=str(time)) for time in times]))
-        self.add_children(bids,'bids')
-        for time in times: self.bid(time).create_variables()
-        #logging.debug('created {} variables {}'.format(str(self),show_clock()))
+        bid_params = dict(
+            owner=self,
+            times=times,
+            input_variable=self.power,
+            min_input=self.Pmin,
+            max_input=self.Pmax,
+            status_variable=self.status,
+            )
+        
+        if self.bid_points is None:
+            bid_params['polynomial'] = self.cost_coeffs
+            bid_params['num_breakpoints'] = self.cost_breakpoints
+        else:
+            bid_params['polynomial'] = None
+            bid_params['bid_points'] = self.bid_points
+
+        self.bids=bidding.Bid(**bid_params)
         return
+
     def create_objective(self,times):
         return sum(self.cost(time) for time in times)
 
@@ -266,8 +280,6 @@ class Generator(OptimizationObject):
             min_down_intervals = roundoff(self.mindowntime/times.intervalhrs)
         
         for t,time in enumerate(times):
-            #bid curve constraints
-            self.bid(time).create_constraints()
             #min/max power
             if self.Pmin>0: self.add_constraint('min gen power', time, self.power(time)>=self.status(time)*self.Pmin)
             self.add_constraint('max gen power', time, self.power(time)<=self.status(time)*self.Pmax)
@@ -315,7 +327,7 @@ class Generator_nonControllable(Generator):
     def __init__(self,
                  schedule=None,
                  power=None,
-                 fuelcost=1,costcurvestring='0',
+                 fuelcost=1,costcurvestring='0',bid_points=None,
                  mustrun=False,
                  Pmin=0,Pmax=None,
                  name='',index=None,bus=None,kind='wind',**kwargs):
@@ -325,24 +337,54 @@ class Generator_nonControllable(Generator):
 
         if Pmax is None: self.Pmax = self.schedule.maxvalue
         self.is_controllable=False
+        self.startupcost=0
+        self.shutdowncost=0
         self.build_cost_model()
         self.init_optimization()
-    def power(self,time): return self.schedule.get_energy(time)
-    def status(self,time): return True
+    def power(self,time=None): return self.schedule.get_energy(time)
+    def status(self,time=None): return True
     def set_initial_condition(self,time=None, P=None, u=None, hoursinstatus=None):
         try: 
             if P is None: P=sorted(self.schedule.energy.items())[0][1] #set initial value to first value
             self.schedule.energy[time]=P 
         except AttributeError: pass #fixed schedule
     def getstatus(self,t,times): return {}
-    def create_variables(self,times): return {}
+    def create_variables(self,times):
+        self.bids=bidding.Bid(
+            polynomial=self.cost_coeffs,
+            owner=self,
+            times=times,
+            fixed_input=True
+            )
     def create_constraints(self,times): return {}
     def cost(self,time,evaluate=False): return self.operatingcost(time)
-    def operatingcost(self,time,evaluate=False): return self.cost_model.output_true( self.power(time) )
+    def operatingcost(self,time=None,evaluate=False): return self.bids.output_true( self.power(time) )
     def truecost(self,time): return self.cost(time)
-    def incrementalcost(self,time): return self.fuelcost*self.cost_model.output_incremental(self.power(time))
+    def incrementalcost(self,time): return self.bids.output_incremental(self.power(time))
 
-        
+class Generator_Stochastic(Generator_nonControllable):
+    """
+    Describes a generator with a stochastic power output.
+    """
+    def __init__(self,
+                 scenario_values=None,
+                 fuelcost=1,costcurvestring='0',bid_points=None,
+                 mustrun=False,
+                 Pmin=0,Pmax=None,
+                 name='',index=None,bus=None,kind='wind',**kwargs):
+        update_attributes(self,locals()) #load in inputs
+        self.is_controllable=False
+        self.is_stochastic=True
+        self.build_cost_model()
+        self.init_optimization()
+    def power(self,time): return self.get_variable('power',time=time,indexed=True)
+    def create_variables(self,times):
+        self.add_parameter('power',index=times.set)
+        power=self.power(time=None)
+        for time in times: power[str(time)]=self.scenario_values[0][time] #initialize to first scenario value
+        return
+    def cost_startup(self,time): return 0
+    def cost_shutdown(self,time): return 0
 
 
 class Load(OptimizationObject):
@@ -372,7 +414,9 @@ class Load(OptimizationObject):
         else: 
             return self.schedule.get_energy(time)
     def shed(self,time,evaluate=False): return self.schedule.get_energy(time) - self.power(time,evaluate)
-    def cost(self,time): return self.cost_shedding*self.shed(time) 
+    def cost(self,time): return self.cost_shedding*self.shed(time)
+    def cost_first_stage(self,times): return 0
+    def cost_second_stage(self,times): return sum(self.cost(time) for time in times)
     def create_variables(self,times):
         if self.shedding_allowed:
             self.add_variable('power',index=times.set,low=0)
@@ -462,7 +506,7 @@ class Bus(OptimizationObject):
         self.generators,self.loads=[],[]
         self.init_optimization()
     
-    def angle(self,time): return self.get_variable('angle',time)
+    def angle(self,time): return self.get_variable('angle',time,indexed=True)
     def price(self,time): return dual(self.get_constraint('power balance',time))
     def Pgen(self,t,evaluate=False):
         if evaluate: return sum(value(gen.power(t)) for gen in self.generators)
@@ -491,12 +535,16 @@ class Bus(OptimizationObject):
         logging.debug('created generator variables {}'.format(show_clock()))
         for load in self.loads: load.create_variables(times)
         logging.debug('created load variables {}'.format(show_clock()))
-        for time in times: self.add_variable('angle',time=time)
+        self.add_variable('angle',index=times.set)
         logging.debug('created bus variables ... returning {}'.format(show_clock()))
         return
-    def create_objective(self,times):
-        return sum(gen.create_objective(times) for gen in self.generators) + \
-            sum(load.create_objective(times) for load in self.loads)
+    def create_objective(self,times): return self.cost_first_stage(times) + self.cost_second_stage(times)
+    def cost_first_stage(self,times):
+        return sum(gen.cost_first_stage(times) for gen in self.generators) + \
+            sum(load.cost_first_stage(times) for load in self.loads)
+    def cost_second_stage(self,times):
+        return sum(gen.cost_second_stage(times) for gen in self.generators) + \
+            sum(load.cost_second_stage(times) for load in self.loads)            
     def create_constraints(self,times,Bmatrix,buses):
         for gen in self.generators: gen.create_constraints(times)
         for load in self.loads: load.create_constraints(times)
@@ -548,15 +596,11 @@ class PowerSystem(OptimizationProblem):
         #add system mode parameters to relevant components
         self.set_load_shedding(load_shedding_allowed) #set load shedding
         for load in loads:
-                try: 
-                    load.cost_model.num_breakpoints=num_breakpoints
-                    load.cost_model.do_segmentation()
+                try: load.cost_breakpoints=num_breakpoints
                 except AttributeError: pass #load has no cost model   
         for gen in generators:
             gen.dispatch_decommit_allowed=dispatch_decommit_allowed
-            try: 
-                gen.cost_model.num_breakpoints=num_breakpoints
-                gen.cost_model.do_segmentation()
+            try: gen.cost_breakpoints=num_breakpoints
             except AttributeError: pass #gen has no cost model
             
     def set_load_shedding(self,is_allowed):
@@ -613,24 +657,22 @@ class PowerSystem(OptimizationProblem):
     def loads(self): return flatten(bus.loads for bus in self.buses)
     def generators(self): return flatten(bus.generators for bus in self.buses)
     def create_variables(self,times):
+        self.add_variable('cost_first_stage')
+        self.add_variable('cost_second_stage')        
         self.add_set('times',[str(t) for t in times])
         times.set=self._model.times
-        
         for bus in self.buses:  bus.create_variables(times)
         for line in self.lines: line.create_variables(times)
         logging.debug('... created power system vars... returning... {}'.format(show_clock()))
         #for var in self.all_variables(times).values(): self.add_variable(var)
-        
-    def create_objective(self,times):
-        obj=sum(bus.create_objective(times) for bus in self.buses) + sum(line.create_objective(times) for line in self.lines)
-        self.add_objective(obj)
+    def cost_first_stage(self): return self.get_component('cost_first_stage')    
+    def cost_second_stage(self): return self.get_component('cost_second_stage')
+    def create_objective(self,times): 
+        self.add_objective(self.cost_first_stage()+self.cost_second_stage())
     def create_constraints(self,times):
         for bus in self.buses: bus.create_constraints(times,self.Bmatrix,self.buses)
         for line in self.lines: line.create_constraints(times,self.buses)
         #a system reserve constraint would go here
-        #for constraint in self.all_constraints(times).values(): self.add_constraint(constraint)
-    
-    
-#def power_to_energy(P,time):
-#    return P*time.intervalhrs
-
+        self.add_constraint('system_cost_first_stage',self.cost_first_stage()==sum(bus.cost_first_stage(times) for bus in self.buses))
+        self.add_constraint('system_cost_second_stage',self.cost_second_stage()==sum(bus.cost_second_stage(times) for bus in self.buses))
+    def iden(self,time): return 'system'
