@@ -10,7 +10,7 @@ from collections import OrderedDict
 from commonscripts import flatten,transpose,elementwiseAdd, getattrL,within,writeCSV,joindir,replace_all,update_attributes
 from schedule import Timelist
 from optimization import value,dual
-import config
+import config,stochastic
 
 for_publication=True
 
@@ -44,6 +44,8 @@ def classify_problem(times,power_system):
     elif len(times)==1: kind='OPF'
     elif not power_system.lines: kind='UC'
     else: kind='SCUC'
+    
+    if any(getattr(g,'is_stochastic',False) for g in power_system.generators()): kind='stochastic_'+kind
     return kind
 
 def make_solution(power_system,times,**kwargs):
@@ -53,7 +55,13 @@ def make_solution(power_system,times,**kwargs):
     :param times: a :class:`~schedule.Timelist` object
     :param power_system: a :class:`~powersystems.PowerSystem` object    
     '''
-    problem_type=dict(ED=Solution_ED, OPF=Solution_OPF, UC=Solution_UC, SCUC=Solution_SCUC)
+    problem_type=dict(
+        ED=Solution_ED, 
+        OPF=Solution_OPF, 
+        UC=Solution_UC, 
+        SCUC=Solution_SCUC,
+        stochastic_UC=Solution_Stochastic_UC
+        )
     kind=classify_problem(times,power_system)
     return problem_type[kind](power_system,times,**kwargs)
 
@@ -70,27 +78,13 @@ class Solution(object):
     '''
     def __init__(self,power_system,times,datadir='.'):
         update_attributes(self,locals())
-        #self.power_system.update_variables()
-        
-        #objgraph.show_backrefs([problem.variables.values()[0]], filename='variable-backref-post-solve.png')
-        #objgraph.show_chain(
-        #    objgraph.find_backref_chain( problem.variables.values()[0],inspect.ismodule),filename='variable-backref-post-solve-module-chain.png')
-        
         self._get_problem_info()
         self._get_costs()
         self._get_prices()
         self._get_outputs()
-        #self.power_system.clear_constraints()
-        #gc.collect()
-        #objgraph.show_backrefs([problem.constraints.values()[0]], filename='constraints-backref-post-solve.png')
-        #objgraph.show_chain(objgraph.find_backref_chain(objgraph.by_type('Constraint')[0],inspect.ismodule),filename='constraint-backref-post-solve-chain.png')
-        #leakers=objgraph.get_leaking_objects()
-        #objgraph.show_most_common_types(objects=leakers)
-        #objgraph.show_refs(leakers[:3], refcounts=True, filename='leakers.png')
     def _get_problem_info(self):
         self.solve_time  =self.power_system.solution_time
         self.objective  =float(value(self.power_system.objective))
-
     def _get_outputs(self):
         self.generators_power =[self.get_values('generators','power',time) for time in self.times]
         self.generators_status=[[s==1 for s in self.get_values('generators','status',time)] for time in self.times]
@@ -180,16 +174,16 @@ class Solution(object):
         ' non-linearized cost of generation={}'.format(self.fuelcost_true_generation),
         'percentage difference\t\t={diff:.2%}'.format(diff=self.costerror),
         ]
-    def save(self,filename=None):
-        if filename is None: filename=joindir(self.datadir,'solution.yaml')
-        def get_time_listing(component,attr):
-            try: return dict(zip(self.times.non_overlap_times,[self.get_values(component,attr,t) for t in self.times.non_overlap_times]))
-            except AttributeError:
-                return dict(zip(self.times,[self.get_values(component,attr,t) for t in self.times]))
-        self.generators_status=get_time_listing('generators','status')
-        self.generators_power=get_time_listing('generators','power')
-        self.power_system=[]
-        with open(filename,'w+') as f: yaml.dump(self,f)
+    # def save(self,filename=None):
+    #     if filename is None: filename=joindir(self.datadir,'solution.yaml')
+    #     def get_time_listing(component,attr):
+    #         try: return dict(zip(self.times.non_overlap_times,[self.get_values(component,attr,t) for t in self.times.non_overlap_times]))
+    #         except AttributeError:
+    #             return dict(zip(self.times,[self.get_values(component,attr,t) for t in self.times]))
+    #     self.generators_status=get_time_listing('generators','status')
+    #     self.generators_power=get_time_listing('generators','power')
+    #     self.power_system=[]
+    #     with open(filename,'w+') as f: yaml.dump(self,f)
 
 
 class Solution_ED(Solution):
@@ -211,12 +205,16 @@ class Solution_ED(Solution):
         ax=plot.axes()
         for gen in generators:
             if gen.status(t):
-                plotted_gens.append( gen.cost_model.plot_derivative(P=value(gen.power(t)),linestyle='-') )
+                in_range,out_range=gen.bids.output_incremental_range()
+                if gen.bids.is_pwl:
+                    line,=plot.step(in_range,out_range,where='pre',linestyle='-')
+                else:
+                    line,=plot.plot(in_range,out_range,linestyle='-', )
+                plotted_gens.append(line)
+                P=value(gen.power(t))
+                IC=gen.incrementalcost(t)
+                plot.plot(P,IC,'.',c=line.get_color(), markersize=8, linewidth=2, alpha=0.7)
                 names_gens.append(gen.name)
-        for load in loads: 
-            if load.kind=='bidding': 
-                plotted_loads.append( load.bid(t).plot_derivative(P=value(load.power(t)),linestyle=':') )
-                names_loads.append(load.name)
         if price is not None: 
             grayColor='.75'
             plot.plot([minGen,maxGen],[price,price],'--k',color=grayColor)
@@ -275,10 +273,10 @@ class Solution_ED(Solution):
         data.append(getattrL(components,'name'))
         
         fields.append('u');   
-        data.append([niceTF(g.status(t)) for g in components])
+        data.append([niceTF(value(g.status(t))) for g in components])
         
         fields.append('P')
-        data.append([nice_zeros(g.power(t)) for g in components])
+        data.append([nice_zeros(value(g.power(t))) for g in components])
         
         fields.append('IC')
         data.append([nice_zeros(g.incrementalcost(t)) for g in components])
@@ -450,6 +448,93 @@ class Solution_UC_multistage(Solution_UC):
     def info_shedding(self):
         return ['total load shed={}MW'.format(self.load_shed) if self.load_shed>0 else '']
 
+
+class Solution_Stochsatic(Solution):
+    def __init__(self,power_system,times,datadir='.'):
+        update_attributes(self,locals())
+        self.scenarios=sorted(self.power_system._scenario_instances.keys())
+        self.power_system._scenario_tree.snapshotSolutionFromInstances(self.power_system._scenario_instances)
+        stochastic.update_variables(self.power_system,times)
+        
+        self._get_problem_info()
+        self._get_costs()
+        self._get_prices()
+        self._get_outputs()
+        
+    def _get_costs(self):
+        instances=self.power_system._scenario_instances
+        tree=self.power_system._scenario_tree
+        root_node = tree._stages[0]._tree_nodes[0]
+        self.expected_cost = root_node.computeExpectedNodeCost(instances)
+        self.cost_per_scenario=stochastic.get_scenario_based_costs(tree,instances)
+    def _get_cost_error(self): pass
+#        try: self.costerror=abs(self.fuelcost_generation-self.fuelcost_true_generation)/self.fuelcost_true_generation
+#        except ZeroDivisionError: self.costerror=0
+    def _get_prices(self): pass
+#        self.lmps={}
+#        self.line_prices={}
+#        for t in self.times: 
+#            self.lmps[str(t)]=self.get_values('buses','price',t)
+#            self.line_prices[str(t)]=self.get_values('lines','price',t)
+    def info_cost(self):
+        return ["expected cost= {}".format(self.expected_cost),
+                "scenario costs: {}".format(self.cost_per_scenario)]
+    def info_generators(self,t):
+        out=['generator info:']
+        if len(self.buses())>1: out.append('bus={}'.format(self.get_values('generators','bus')))
+        out.extend(['name={}'.format(self.get_values('generators','name')),
+                    'u={}'.format(self.get_values('generators','status',t))])
+        return out
+    def show(self):
+        '''Display the solution information to the terminal'''
+        #self.problem.scenario_tree.pprintSolution()
+        out=['']
+        out.extend(['Solution information','-'*20])
+        out.extend(self.info_cost())
+        
+        for t in self.times:
+                out.append('{tm}: {start}'.format(tm=t,start=t.Start))
+                out.extend(self.info_generators(t))
+        print '\n'.join(out)
+        #P=self.generators()[2].power(self.times[0])
+        #print P.name,values
+
+    def get_values(self,kind='generators',attrib='power',time=None,scenario=None):
+        '''Get the attributes of all objects of a certain kind at a given time and scenario.'''
+        method={'generators':self.generators,'loads':self.loads,'lines':self.lines,'buses':self.buses}
+        if time is not None:
+            out=[]
+            for obj in method[kind]():
+                var=getattr(obj, attrib)(time)
+                var.pprint()
+                #self.power_system.show_model()
+                self.power_system._stochastic_instance.pprint()
+                barf
+                try: out.append(var)
+                except AttributeError: out.append(var)
+                except KeyError: out.append(var.name)
+            return out
+        else: return [getattr(obj, attrib) for obj in method[kind]()]
+    def value(self,var,scenario): 
+        try: return self.power_system.variables[var.name][scenario]
+        except AttributeError: var
+class Solution_Stochastic_UC(Solution_Stochsatic):
+    def saveCSV(self,filename=None):
+        '''generator power values and statuses for unit commitment'''
+        if filename is None: filename=joindir(self.datadir,'commitment.csv')
+        data=[]
+        fields=['generators','times','scenarios','power','status']
+        for gen in self.generators():
+            for time in self.times:
+                for scenario in self.scenarios:
+                    try: row=[gen.name,str(time.Start),scenario,self.value(gen.power(time),scenario),self.value(gen.status(time),scenario)]
+                    except KeyError:
+                        print gen.power(time) 
+                        raise
+                    data.append(row)
+        writeCSV(fields,data,filename=filename)
+
+
 def _colormap(numcolors,colormapName='gist_rainbow',mincolor=1):
     cm = matplotlib.cm.get_cmap(colormapName)
     return [cm(1.*i/numcolors) for i in range(mincolor,numcolors+mincolor)]      
@@ -581,4 +666,4 @@ def prettify_axes(ax):
     ax.get_yaxis().tick_left()
 def shrink_axis(ax,percent_horizontal=0.20,percent_vertical=0):
     box = ax.get_position()
-    ax.set_position([box.x0, box.y0, box.width * (1-percent_horizontal), box.height*(1-percent_vertical)])
\ No newline at end of file
+    ax.set_position([box.x0, box.y0, box.width * (1-percent_horizontal), box.height*(1-percent_vertical)])
